@@ -75,6 +75,7 @@ class APLRRegressor:
         penalty_for_interactions: float = 0.0,
         max_terms: int = 0,
         ridge_penalty: float = 0.0001,
+        mean_bias_correction: bool = False,
     ):
         self.m = m
         self.v = v
@@ -122,6 +123,7 @@ class APLRRegressor:
         self.penalty_for_interactions = penalty_for_interactions
         self.max_terms = max_terms
         self.ridge_penalty = ridge_penalty
+        self.mean_bias_correction = mean_bias_correction
 
         # Creating aplr_cpp and setting parameters
         self.APLRRegressor = aplr_cpp.APLRRegressor()
@@ -183,6 +185,7 @@ class APLRRegressor:
         self.APLRRegressor.penalty_for_interactions = self.penalty_for_interactions
         self.APLRRegressor.max_terms = self.max_terms
         self.APLRRegressor.ridge_penalty = self.ridge_penalty
+        self.APLRRegressor.mean_bias_correction = self.mean_bias_correction
 
     def fit(
         self,
@@ -315,6 +318,115 @@ class APLRRegressor:
     def set_intercept(self, value: float):
         self.APLRRegressor.set_intercept(value)
 
+    def plot_affiliation_shape(
+        self,
+        affiliation: str,
+        plot: bool = True,
+        save: bool = False,
+        path: str = "",
+    ):
+        """
+        Plots or saves the shape of a given unique term affiliation.
+
+        For main effects, it produces a line plot. For two-way interactions, it produces a heatmap.
+        Plotting for higher-order interactions is not supported.
+
+        :param affiliation: A string specifying which unique_term_affiliation to use.
+        :param plot: If True, displays the plot.
+        :param save: If True, saves the plot to a file.
+        :param path: The file path to save the plot. If empty and save is True, a default path will be used.
+        """
+        try:
+            import matplotlib.pyplot as plt
+        except ImportError:
+            raise ImportError("matplotlib is required for plotting. Please install it.")
+
+        all_affiliations = self.get_unique_term_affiliations()
+        if affiliation not in all_affiliations:
+            raise ValueError(
+                f"Affiliation '{affiliation}' not found in model. "
+                f"Available affiliations are: {all_affiliations}"
+            )
+
+        affiliation_index = all_affiliations.index(affiliation)
+
+        predictors_in_each_affiliation = (
+            self.get_base_predictors_in_each_unique_term_affiliation()
+        )
+        predictor_indexes_used = predictors_in_each_affiliation[affiliation_index]
+
+        shape = self.get_unique_term_affiliation_shape(affiliation)
+        if shape.shape[0] == 0:
+            print(f"No shape data available for affiliation '{affiliation}'.")
+            return
+
+        predictor_names = affiliation.split(" & ")
+
+        is_main_effect: bool = len(predictor_indexes_used) == 1
+        is_two_way_interaction: bool = len(predictor_indexes_used) == 2
+
+        if is_main_effect:
+            fig = plt.figure()
+            # Sort by predictor value for a clean line plot
+            sorted_indices = np.argsort(shape[:, 0])
+            plt.plot(shape[sorted_indices, 0], shape[sorted_indices, 1])
+            plt.xlabel(predictor_names[0])
+            plt.ylabel("Contribution to linear predictor")
+            plt.title(f"Main effect of {predictor_names[0]}")
+            plt.grid(True)
+        elif is_two_way_interaction:
+            fig = plt.figure(figsize=(8, 6))
+
+            # Get unique coordinates and their inverse mapping
+            y_unique, y_inv = np.unique(shape[:, 0], return_inverse=True)
+            x_unique, x_inv = np.unique(shape[:, 1], return_inverse=True)
+
+            # Create grid for sums and counts
+            grid_sums = np.zeros((len(y_unique), len(x_unique)))
+            grid_counts = np.zeros((len(y_unique), len(x_unique)))
+
+            # Populate sums and counts to later calculate the mean
+            np.add.at(grid_sums, (y_inv, x_inv), shape[:, 2])
+            np.add.at(grid_counts, (y_inv, x_inv), 1)
+
+            # Calculate mean, avoiding division by zero
+            with np.errstate(divide="ignore", invalid="ignore"):
+                pivot_table_values = np.true_divide(grid_sums, grid_counts)
+                # Where there's no data, pivot_table_values will be nan, which is fine for imshow.
+
+            plt.imshow(
+                pivot_table_values,
+                aspect="auto",
+                origin="lower",
+                extent=[
+                    x_unique.min(),
+                    x_unique.max(),
+                    y_unique.min(),
+                    y_unique.max(),
+                ],
+                cmap="Blues_r",
+            )
+            plt.colorbar(label="Contribution to the linear predictor")
+            plt.xlabel(predictor_names[1])
+            plt.ylabel(predictor_names[0])
+            plt.title(
+                f"Interaction between {predictor_names[0]} and {predictor_names[1]}"
+            )
+        else:
+            print(
+                f"Plotting for interaction level > 2 is not supported. Affiliation: {affiliation}"
+            )
+            return
+
+        if save:
+            save_path = path or f"shape_of_{affiliation.replace(' & ', '_')}.png"
+            plt.savefig(save_path)
+
+        if plot:
+            plt.show()
+
+        plt.close(fig)
+
     def remove_provided_custom_functions(self):
         self.APLRRegressor.remove_provided_custom_functions()
         self.calculate_custom_validation_error_function = None
@@ -356,6 +468,7 @@ class APLRRegressor:
             "penalty_for_interactions": self.penalty_for_interactions,
             "max_terms": self.max_terms,
             "ridge_penalty": self.ridge_penalty,
+            "mean_bias_correction": self.mean_bias_correction,
         }
 
     # For sklearn
@@ -504,7 +617,36 @@ class APLRClassifier:
         return self.APLRClassifier.get_categories()
 
     def get_logit_model(self, category: str) -> APLRRegressor:
-        return self.APLRClassifier.get_logit_model(category)
+        logit_model_cpp = self.APLRClassifier.get_logit_model(category)
+
+        logit_model_py = APLRRegressor(
+            m=self.m,
+            v=self.v,
+            random_state=self.random_state,
+            loss_function="binomial",
+            link_function="logit",
+            n_jobs=self.n_jobs,
+            cv_folds=self.cv_folds,
+            bins=self.bins,
+            max_interaction_level=self.max_interaction_level,
+            max_interactions=self.max_interactions,
+            min_observations_in_split=self.min_observations_in_split,
+            ineligible_boosting_steps_added=self.ineligible_boosting_steps_added,
+            max_eligible_terms=self.max_eligible_terms,
+            verbosity=self.verbosity,
+            boosting_steps_before_interactions_are_allowed=self.boosting_steps_before_interactions_are_allowed,
+            monotonic_constraints_ignore_interactions=self.monotonic_constraints_ignore_interactions,
+            early_stopping_rounds=self.early_stopping_rounds,
+            num_first_steps_with_linear_effects_only=self.num_first_steps_with_linear_effects_only,
+            penalty_for_non_linearity=self.penalty_for_non_linearity,
+            penalty_for_interactions=self.penalty_for_interactions,
+            max_terms=self.max_terms,
+            ridge_penalty=self.ridge_penalty,
+        )
+
+        logit_model_py.APLRRegressor = logit_model_cpp
+
+        return logit_model_py
 
     def get_validation_error_steps(self) -> FloatMatrix:
         return self.APLRClassifier.get_validation_error_steps()
